@@ -27,6 +27,9 @@
 #include <QStandardItemModel>
 #include <QItemSelection>
 #include <QNetworkAddressEntry>
+#include <QFormLayout>
+#include <QSpinBox>
+#include <QHostInfo>
 
 #include <KEditListWidget>
 #include <KLocalizedString>
@@ -82,7 +85,7 @@ IPv4Widget::IPv4Widget(const NetworkManager::Setting::Ptr &setting, QWidget* par
     m_ui->setupUi(this);
 
     m_ui->tableViewAddresses->setModel(&d->model);
-    m_ui->tableViewAddresses->horizontalHeader()->setResizeMode(QHeaderView::Interactive);
+    m_ui->tableViewAddresses->horizontalHeader()->setSectionResizeMode(QHeaderView::Interactive);
     m_ui->tableViewAddresses->horizontalHeader()->setStretchLastSection(true);
 
     IpV4Delegate *ipDelegate = new IpV4Delegate(this);
@@ -108,6 +111,7 @@ IPv4Widget::IPv4Widget(const NetworkManager::Setting::Ptr &setting, QWidget* par
     slotModeComboChanged(m_ui->method->currentIndex());
 
     connect(m_ui->btnRoutes, &QPushButton::clicked, this, &IPv4Widget::slotRoutesDialog);
+    connect(m_ui->btnAdvanced, &QPushButton::clicked, this, &IPv4Widget::slotAdvancedDialog);
 
     // Connect for setting check
     watchChangedSetting();
@@ -140,6 +144,10 @@ void IPv4Widget::loadConfig(const NetworkManager::Setting::Ptr &setting)
     m_tmpIpv4Setting.setNeverDefault(ipv4Setting->neverDefault());
     m_tmpIpv4Setting.setIgnoreAutoRoutes(ipv4Setting->ignoreAutoRoutes());
 
+    m_tmpIpv4Setting.setDhcpHostname(ipv4Setting->dhcpHostname());
+    m_tmpIpv4Setting.setDhcpSendHostname(ipv4Setting->dhcpSendHostname());
+    m_tmpIpv4Setting.setDadTimeout(ipv4Setting->dadTimeout());
+
     // method
     switch (ipv4Setting->method()) {
         case NetworkManager::Ipv4Setting::Automatic:
@@ -200,6 +208,10 @@ QVariantMap IPv4Widget::setting() const
     ipv4Setting.setNeverDefault(m_tmpIpv4Setting.neverDefault());
     ipv4Setting.setIgnoreAutoRoutes(m_tmpIpv4Setting.ignoreAutoRoutes());
 
+    ipv4Setting.setDhcpHostname(m_tmpIpv4Setting.dhcpHostname());
+    ipv4Setting.setDhcpSendHostname(m_tmpIpv4Setting.dhcpSendHostname());
+    ipv4Setting.setDadTimeout(m_tmpIpv4Setting.dadTimeout());
+
     // method
     switch ((MethodIndex)m_ui->method->currentIndex()) {
         case Automatic:
@@ -416,6 +428,50 @@ void IPv4Widget::slotRoutesDialog()
     dlg->show();
 }
 
+void IPv4Widget::slotAdvancedDialog()
+{
+    auto dlg = new QDialog(this);
+    dlg->setAttribute(Qt::WA_DeleteOnClose);
+
+    auto layout = new QFormLayout(dlg);
+    dlg->setLayout(layout);
+
+    layout->addRow(new QLabel(i18n("<qt>You can find more information about these values here:<br/><a href='https://developer.gnome.org/NetworkManager/stable/nm-settings.html'>https://developer.gnome.org/NetworkManager/stable/nm-settings.html</a></qt>")));
+
+    auto sendHostname = new QCheckBox(dlg);
+    sendHostname->setChecked(m_tmpIpv4Setting.dhcpSendHostname());
+    layout->addRow(i18n("Send hostname:"), sendHostname);
+
+    auto dhcpHostname = new QLineEdit(dlg);
+    dhcpHostname->setText(m_tmpIpv4Setting.dhcpHostname());
+    dhcpHostname->setPlaceholderText(QHostInfo::localHostName());
+    layout->addRow(i18n("DHCP hostname:"), dhcpHostname);
+
+    connect(sendHostname, &QCheckBox::toggled, dhcpHostname, &QLineEdit::setEnabled);
+
+    auto dadTimeout = new QSpinBox(dlg);
+    dadTimeout->setSpecialValueText(i18n("Default"));
+    dadTimeout->setSuffix(i18nc("Milliseconds", " ms"));
+    dadTimeout->setMinimum(-1);
+    dadTimeout->setValue(m_tmpIpv4Setting.dadTimeout());
+    layout->addRow(i18n("DAD timeout:"), dadTimeout);
+
+    QDialogButtonBox* box = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel, dlg);
+    connect(box, &QDialogButtonBox::accepted, dlg, &QDialog::accept);
+    connect(box, &QDialogButtonBox::rejected, dlg, &QDialog::reject);
+    layout->addWidget(box);
+
+    connect(dlg, &QDialog::accepted, this,
+            [=] () {
+                m_tmpIpv4Setting.setDhcpSendHostname(sendHostname->isChecked());
+                m_tmpIpv4Setting.setDhcpHostname(dhcpHostname->text());
+                m_tmpIpv4Setting.setDadTimeout(dadTimeout->value());
+            });
+
+    dlg->setModal(true);
+    dlg->show();
+}
+
 void IPv4Widget::slotDnsServers()
 {
     QPointer<QDialog> dialog = new QDialog(this);
